local mod = RW:Boss("The Lich King", "Icecrown Citadel", "Icecrown Citadel", "The Frozen Throne")
local L

mod.BossIcon = [[Interface\Icons\achievement_boss_lichking]]
mod.SubCatIcon = [[Interface\Icons\achievement_dungeon_frozenthrone]]

------------------------------------
-- Variables
local mPhase, tActive

-- Shambling Horror
local tShamblingCD, wShambling

-- Infest
local tInfestCD, tInfestCast, wInfest

-- Necrotic Plague
local tNecroticCD, wNecrotic, mNecrotic

-- Soul Reaper
local tReaperCD

-- Defile
local tDefileCD, wDefileCast, wDefileRun

-- Valkyr
local wValkyr

-----------------------------------
-- Init

function mod:OnRegister()
	L = mod:RegisterLocale("enUS")
	if L then
		L.name = "The Lich King"
		L.YellTrigger = "So the Light's vaunted justice has finally arrived? Shall I lay down Frostmourne and throw myself at your mercy, Fordring?"
		
		L.YellValkyr = "Val'kyr, your master calls!"
		
		L.defile_on_player = "Defile on me!"
		
		L.active = " active"
		L.active_opt = "Timer for when the fight starts"
		L.active_grup = "Fight start"
		
		L.new_valkyr = "Val'kyr up!"
		L.new_valkyr_opt = "Warn when a new Val'kyr is spawned"
		L.valkyr_group = "Val'Kyrs"
	end
	L = mod:GetLocale()
	mod.L = L
	mod:SetTrigger(36597, L.YellTrigger)
	mod:SetWin()
	mod:SetDelay(60)
	
	mPhase 				= self:PhaseMessage()
	tActive				= self:CustomTimer(58, L.name..L.active, L.active_opt, [[Interface\Icons\achievement_boss_lichking]],  L.active_grup)
	
	-- Shambling Horror
	tShamblingCD		= self:CDTimer(60, 70372)
	wShambling			= self:Warning(70372)
	
	-- Infest
	tInfestCD			= self:CDTimer(20, 70541)
	tInfestCast			= self:CastTimer(70541)
	wInfest				= self:Warning(70541)
	
	-- Necrotic Plague
	tNecroticCD			= self:CDTimer(30, 70337)
	mNecrotic			= self:TargetMessage(70337)
	wNecrotic			= self:RunawayWarning(70337)
	
	-- Soul Reaper
	tReaperCD			= self:CDTimer(30, 69409)
	
	-- Defile
	tDefileCD			= self:CDTimer(30, 72762)
	wDefileCast			= self:CastWarning(72762)
	wDefileRun			= self:RunawayWarning(72762)
	
	-- Valkyr
	wValkyr				= self:CustomWarning(L.new_valkyr, L.new_valkyr_opt, [[Interface\Icons\achievement_boss_svalasorrowgrave]], L.valkyr_group)
end

-----------------------------------------
-- Locals
local defileSpamm

-----------------------------------------
-- Start

function mod:OnEngage(trigger)
	self:Track("ShamblingHorror", 70372)
	self:Track("Infest", 70541, 73779, 73780, 73781)
	self:Track("NercroticPlague", 70337)
	self:Track("SoulReaper", 69409)
	self:Track("Defile", 72762, 72754, 73708, 73709, 73710) -- First is the spell casted by arthas to summon the Defile
	self:Track("Quake", 72262)
	
	self:YellTrack("Valkyr", L.YellValkyr)
	
	if  trigger == L.YellTrigger then
		tActive:Start()
	end
	
	defileSpamm = 0
end

function mod:ShamblingHorror(event, args)
	if event == "SPELL_SUMMON" then
		tShamblingCD:Start()
		wShambling:Show()
	end
end

function mod:Infest(event, args)
	if event == "SPELL_CAST_START" then
		tInfestCD:Start()
		tInfestCast:Start()
		wInfest:Show()
	end
end

function mod:NercroticPlague(event, args)
	if event == "SPELL_CAST_SUCCESS" then
		if args:IsPlayer() then
			wNecrotic:Show()
		end
		tNecroticCD:Start()
		mNecrotic:Show(args.dName)
	
	elseif event == "SPELL_DISPELL" then
		
	end
end

function mod:SoulReaper(event, args)
	if event == "SPELL_CAST_SUCCESS" then
		tReaperCD:Start()
	end
end

local function ScanDefileTarget()
	local target = mod:GetMobTarget(36597)
	if target then
		if target == mod.pName then
			SendChatMessage(L.defile_on_player, "SAY")
			
		elseif self.Party.group[target] then
			local uID = self.Party.group[target].id
			if uID and CheckInteractDistance(uID, 2) then
				wDefileRun:Show()
			end
		end
	end
end

function mod:Defile(event, args)
	if event == "SPELL_CAST_START" then
		tDefileCD:Start()
		wDefileCast:Show()
		self:Schedule("DefileScan", 0.1, ScanDefileTarget)
		
	elseif event == "SPELL_DAMAGE" then
		if args:IsPlayer() and GetTime() - defileSpamm > 2 then
			wDefileRun:Show()
			defileSpamm = GetTime()
		end
	end
end

function mod:Quake(event, args)
	if event == "SPELL_CAST_START" then
		mPhase:Show(2)
	end
end

function mod:Valkyr(event, msg, name)
	wValkyr:Show()
end