
local RW = LibStub("AceAddon-3.0"):GetAddon("Raid Watch")
local plugin = RW:Plugin("Direction", "AceConsole-3.0")

----------------------------
-- Locals

-- local LP = LibStub("AceLocale-3.0"):GetLocale("RW-2.0-Plugins")
local LSM = LibStub("LibSharedMedia-3.0")
local Party
local db

local anchor

local AllArrows = {}
local UnusedArrows = {}
local UsedArrows = {}
local UsedArrowsI = {}

local arrowCount = 0

local iconCoords = {
	{0, 	0.25,	0, 		0.25},
	{0.25,	0.5,	0,		0.25},
	{0.5, 	0.75,	0,		0.25},
	{0.75,	1,		0,		0.25},
	{0,		0.25,	0.25,	0.5},
	{0.25,	0.5,	0.25,	0.5},
	{0.5,	0.75,	0.25,	0.5},
	{0.75,	1,		0.25,	0.5},
}
local iconTexture = "Interface\\TargetingFrame\\UI-RaidTargetingIcons.blp"

local ExpandList = {
	{value=1,	text="Left",	icon = [[Interface\MoneyFrame\Arrow-Left-Up]]},
	{value=2, text="Right",	icon = [[Interface\MoneyFrame\Arrow-Right-Up]]},
	{value=3, 	text="Up",		icon = [[Interface\Vehicles\UI-Vehicles-Button-Pitch-Up]]},
	{value=4, 	text="Down",	icon = [[Interface\Vehicles\UI-Vehicles-Button-PitchDown-Up]]},
	{value=5, text="Center Horizontally",	icon = [[Interface\PAPERDOLLINFOFRAME\UI-GearManager-Undo]]},
	{value=6, text="Center Vertically",	icon = [[Interface\PAPERDOLLINFOFRAME\UI-GearManager-Undo]]},
}
------
--Math
local PI = math.pi
local PI2 = math.pi * 2

local defaults = {
	profile = {
		PluginEnabled = true,
		x = 0,
		y = 0,
		ExpandDir = 5,
		Width = 160,
		Height = 160,
		Color = {1, 1, 1},
		ColorClose = {0, 1, 0},
		ColorMiddle = {0.9, 0.65, 0.01},
		ColorFar = {1, 0, 0},
		Font = RW.defaults.profile["Font"],
		FontSize = 18,
		FontColor = RW.defaults.profile["FontColor"],
	}
}

----------------------------
-- Init

function plugin:OnRegister()
	self.db = RW.db:RegisterNamespace("Direction", defaults)
	db = self.db.profile
	
	self.db.RegisterCallback(self, "OnProfileChanged", "Reset")
	self.db.RegisterCallback(self, "OnProfileCopied", "Reset")
	self.db.RegisterCallback(self, "OnProfileReset", "Reset")
end

function plugin:PluginEnable()
	if not db.PluginEnabled then
		self:Disable()
		return
	end	
	
	self:RegisterMessage("RW_ArrowToNameOrUnit")
	self:RegisterMessage("RW_ArrowToPosition")
	self:RegisterMessage("RW_HideAllArrows")
	self:RegisterMessage("RW_HideArrow")
	self:RegisterMessage("RW_ConfigMode")
	
	self:CreateAnchor("Direction Guides")
	
	self:RegisterChatCommand("dir", "SlashHandler")
	
	self:Reset()
	
	Party = RW:GetPlugin("Party")
end

function plugin:PluginDisable()
	if self.frame then
		self.frame:Hide()
	end
end

function plugin:Reset()
	db = self.db.profile
	
	local scale = anchor:GetEffectiveScale() / UIParent:GetEffectiveScale()
	anchor:ClearAllPoints()
	anchor:SetPoint("CENTER", UIParent, "CENTER", db.x / scale, db.y / scale)		
	anchor:SetWidth(db.Width)
	
	for i, arrow in pairs(AllArrows) do
		arrow.t:SetVertexColor(unpack(db.Color))
		arrow.name:SetFont(LSM:Fetch("font", db.Font), db.FontSize, "OUTLINE")
		arrow.name:SetTextColor(unpack(db.FontColor))
		arrow.extrainfo:SetFont(LSM:Fetch("font", db.Font), db.FontSize, "OUTLINE")
		arrow.extrainfo:SetTextColor(unpack(db.FontColor))
		arrow:SetHeight(db.Height)
		arrow:SetWidth(db.Width)
		if (db.Height - 60) < db.Width then
			arrow.t:SetWidth(db.Height - 60)
			arrow.t:SetHeight(db.Height - 60)
		else
			arrow.t:SetWidth(db.Width)
			arrow.t:SetHeight(db.Width)
		end
	end
	
	self:ArrangeArrows()
end

local function dragStart(self)
	self:StartMoving()
end
local function dragStop(self)
	self:StopMovingOrSizing()
	local scale = self:GetEffectiveScale() / UIParent:GetEffectiveScale()
	
	local x, y = self:GetCenter()
	x, y = x * scale, y * scale
	
	x = x - GetScreenWidth()/2
	y = y - GetScreenHeight()/2
	
	db.x = x
	db.y = y
	
	plugin:SendMessage("RW_GUIValuesChanged")
end

function plugin:CreateAnchor(name)
	local nanchor = RW:CreateAnchor(name, UIParent, name)	
	nanchor:SetScript("OnDragStart", dragStart)
	nanchor:SetScript("OnDragStop", dragStop)
	nanchor.name = name
	
	nanchor.arrows = {}
	
	anchor = nanchor
end

local function Click(self, button)
	if button == "RightButton" and IsShiftKeyDown() then
		plugin:HideArrow(self)
		RW:TRemove(UsedArrowsI, self)
		plugin:ArrangeArrows()
	end
end

function plugin:GetArrow()
	local arrow = table.remove(UnusedArrows)
	if arrow then return arrow end
	
	arrow = CreateFrame("Frame", "RWDirectionArrow"..(#AllArrows + 1), UIParent)
	arrow:SetHeight(db.Height)
	arrow:SetWidth(db.Width)
	arrow:SetScript("OnMouseDown", Click)
	arrow:RegisterEvent("MODIFIER_STATE_CHANGED")
	
	arrow:SetScript("OnEvent", function(self, event, arg1, arg2)
		if arg1 == "LSHIFT" or arg1 == "RSHIFT" then
			if arg2 == 1 then
				self:EnableMouse(true)
				self:SetScript("OnMouseDown", Click)
			else
				self:EnableMouse(false)
				self:SetScript("OnMouseDown", nil)
			end
		end
	end)
	
	local icon = arrow:CreateTexture()
	icon:SetTexture(iconTexture)
	icon:SetPoint("BOTTOMLEFT")
	icon:SetHeight(30)
	icon:SetWidth(30)
	icon:Hide()
	arrow.icon = icon
	
	local namestring = arrow:CreateFontString("RWDirectionArrowName"..(#AllArrows + 1), "ARTWORK")
	namestring:SetPoint("BOTTOMLEFT", icon, "BOTTOMRIGHT", 5, 0)
	namestring:SetPoint("BOTTOMRIGHT")
	namestring:SetJustifyH("LEFT")
	namestring:SetHeight(30)
	namestring:SetFont(LSM:Fetch("font", db.Font), db.FontSize, "OUTLINE")
	namestring:SetTextColor(unpack(db.FontColor))
	arrow.name = namestring
	
	local extraicon = arrow:CreateTexture()
	extraicon:SetPoint("TOPLEFT")
	extraicon:SetHeight(30)
	extraicon:SetWidth(30)
	extraicon:Hide()
	arrow.extraicon = extraicon
		
	local extrainfo = arrow:CreateFontString("RWDirectionArrowInfo"..(#AllArrows + 1), "ARTWORK")
	extrainfo:SetPoint("TOPLEFT", extraicon, "TOPRIGHT", 5, 0)
	extrainfo:SetPoint("TOPRIGHT")
	extrainfo:SetJustifyH("LEFT")
	extrainfo:SetHeight(30)
	extrainfo:SetFont(LSM:Fetch("font", db.Font), db.FontSize, "OUTLINE")
	extrainfo:SetTextColor(unpack(db.FontColor))
	arrow.extrainfo = extrainfo
	
	local texture = arrow:CreateTexture()
	texture:SetTexture([[Interface\AddOns\RW_Core\Media\Arrow.tga]])
	texture:SetPoint("CENTER")
	texture:SetWidth(db.Height - 60)
	texture:SetVertexColor(unpack(db.Color))
	arrow.t = texture
		
	local AnimGroup = texture:CreateAnimationGroup()
	local Rotation = AnimGroup:CreateAnimation("Rotation")
	local AnimHolder = AnimGroup:CreateAnimation("Rotation")
	
	Rotation:SetDegrees(0.0)
	Rotation:SetDuration(0)
	
	AnimHolder:SetDegrees(0)
	AnimHolder:SetDuration(999999999)
	
	arrow.AnimGroup = AnimGroup
	arrow.Rotation = Rotation
	arrow.AnimHolder = AnimHolder
	
	tinsert(AllArrows, arrow)
	
	if (db.Height - 60) < db.Width then
		arrow.t:SetWidth(db.Height - 60)
		arrow.t:SetHeight(db.Height - 60)
	else
		arrow.t:SetWidth(db.Width)
		arrow.t:SetHeight(db.Width)
	end
	
	return arrow
end

function plugin:ArrangeArrows()
	for i, arrow in pairs(UsedArrowsI) do
		arrow:ClearAllPoints()
		if i == 1 then
			if db.ExpandDir == 1 then
				arrow:SetPoint("BOTTOMRIGHT", anchor, "TOPRIGHT")
			elseif db.ExpandDir == 4 then
				arrow:SetPoint("TOPLEFT", anchor, "BOTTOMLEFT")
			else
				arrow:SetPoint("BOTTOMLEFT", anchor, "TOPLEFT")
			end
		else
			if db.ExpandDir == 2 then
				arrow:SetPoint("BOTTOMLEFT", UsedArrowsI[i-1], "BOTTOMRIGHT")
			elseif db.ExpandDir == 1 then
				arrow:SetPoint("BOTTOMRIGHT", UsedArrowsI[i-1], "BOTTOMLEFT")
			elseif db.ExpandDir == 3 then
				arrow:SetPoint("BOTTOMLEFT", UsedArrowsI[i-1], "TOPLEFT")
			elseif db.ExpandDir == 4 then
				arrow:SetPoint("TOPLEFT", UsedArrowsI[i-1], "BOTTOMLEFT")
			elseif db.ExpandDir == 5 then
				arrow:SetPoint("BOTTOMLEFT", UsedArrowsI[i-1], "BOTTOMRIGHT")
				UsedArrowsI[1]:SetPoint("BOTTOMLEFT", anchor, "TOPLEFT", -(#UsedArrowsI * db.Width / 2) + db.Width / 2, 0)
			elseif db.ExpandDir == 6 then
				arrow:SetPoint("BOTTOMLEFT", UsedArrowsI[i-1], "TOPLEFT")
				local diff = -(anchor:GetHeight() / 2) - (#UsedArrowsI) * (db.Height / 2)
				UsedArrowsI[1]:SetPoint("BOTTOMLEFT", anchor, "TOPLEFT", 0, diff)
			end
		end
	end
end

function plugin:HideArrow(arrow)
	arrow:Hide()
	UsedArrows[arrow.UserData.id] = nil
	wipe(arrow.UserData)
	tinsert(UnusedArrows, arrow)
end

function plugin:SetArrowAngle(arrow, dx, dy)
	local pf = GetPlayerFacing()
	local px, py = math.sin(pf), math.cos(pf)
	local angle = math.atan2(dx, dy) - math.atan2(px, py)
	
	if arrow.UserData.away then angle = angle + PI end
	
	arrow.Rotation:SetRadians(angle)
	arrow.AnimGroup:Play()
end

local function ArrowUpdate(self, elapsed)
	local d, dx, dy	
	
	if self.UserData.trackType == "UNIT" then	
		d, dx, dy = RW:GetDistanceToUnit(self.UserData.unit)
		if not d or not dx or not dy then
			local px, py = GetPlayerMapPosition("player")
			local mx, my = GetPlayerMapPosition(self.UserData.unit)
			if not mx or not my then
				plugin:HideArrow(self)
				return
			end
			dx = px - mx
			dy = py - my
			
			local c = self.UserData.classcolor
			self.name:SetText(("|cff%2x%2x%2x%s|r"):format(c.r * 255, c.g * 255, c.b * 255, self.UserData.name))
		else			
			local c = self.UserData.classcolor
			self.name:SetText(("|cff%2x%2x%2x%s|r (%dy)"):format(c.r * 255, c.g * 255, c.b * 255, self.UserData.name, d))
		end
		
	
		local raidtarget = GetRaidTargetIndex(self.UserData.unit)
		if raidtarget then
			self.icon:SetTexCoord(unpack(iconCoords[raidtarget]))
			self.icon:Show()
		else
			self.icon:Hide()
		end
		
		if self.UserData.extraicon then
			self.extraicon:SetTexture(self.UserData.extraicon)
			self.extraicon:SetTexCoord(0.02, 0.98, 0.02, 0.98)
			self.extraicon:Show()
		else
			self.extraicon:Hide()
		end
		
	elseif self.UserData.trackType == "POSITION" then
		local dims = RW:GetMapDims()
		local px, py = GetPlayerMapPosition("player")
		local mx, my = self.UserData.x, self.UserData.y
		if dims then			
			px, py = px * dims.w, py * dims.h
			mx, my = mx * dims.w, my * dims.h
			dx = px - mx
			dy = py - my
			d = math.sqrt(dx * dx + dy * dy)
			self.name:SetText(("[%.0f, %.0f] (%dy)"):format(mx, my, d))
		else
			dx = px - mx
			dy = py - my
			self.name:SetText(("[%.2f, %.2f]"):format(mx, my))
		end
	end
	
	if self.UserData.extra then
		self.extrainfo:SetText(self.UserData.extra..(self.UserData.away and " ("..self.UserData.away.."y)" or ""))
		self.extrainfo:Show()
	else
		self.extrainfo:Hide()
	end
	
	if not dx or not dy then
		plugin:HideArrow(self)
		return
	end
	
	local r1, g1, b1 = unpack(db.ColorClose)
	local r2, g2, b2 = unpack(db.ColorMiddle)
	local r3, g3, b3 = unpack(db.ColorFar)
	local p = 0.5
	if self.UserData.away then
		if d then
			p = d/self.UserData.away
		end
		self.t:SetVertexColor(RW:ColorGradient(p, r3, g3, b3, r2, g2, b2, r1, g1, b1))
	else
		if d then
			p = d/40
		end
		self.t:SetVertexColor(RW:ColorGradient(p, r1, g1, b1, r2, g2, b2, r3, g3, b3))
	end
	if d then
		dx = dx / d
		dy = dy / d
	end
	plugin:SetArrowAngle(self, dx, dy)
end

function plugin:InitArrow(arrow, id, away, extra, icon)
	arrow.UserData = arrow.UserData or {}
	
	local spellName, spellIcon
	if type(extra) == "number" then
		spellName, _, spellIcon = GetSpellInfo(extra)
	end
	
	arrow.UserData.id = id
	arrow.UserData.away = away
	arrow.UserData.extra = spellName or extra
	arrow.UserData.extraicon = icon or spellIcon
	arrow:SetScript("OnUpdate", ArrowUpdate)	
	arrow:Show()
	
	UsedArrows[id] = arrow
	tinsert(UsedArrowsI, arrow)	
	self:ArrangeArrows()
end

function plugin:RW_ArrowToNameOrUnit(message, id, uID, tName, away, extra, icon)
	if UsedArrows[id] then return end
	
	--[===[@debug@
	if type(away) ~= "number" then
		error("Away number must be of type number")
	end
	--@end-debug@]===]
	
	local unit
	if uID and UnitExists(uID) then
		unit = uID
	else
		for n, data in Party:PartyIterator() do
			if n == tName then
				unit = data.id
			end
		end
	end
	if not unit then return end
	if UnitIsUnit(unit, "player") then return end
	
	local arrow = self:GetArrow()
	if not arrow then return end
	
	self:InitArrow(arrow, id, away, extra, icon)
	
	arrow.UserData.classcolor = RAID_CLASS_COLORS[select(2, UnitClass(unit))]
	arrow.UserData.name = UnitName(unit)
	arrow.UserData.trackType = "UNIT"
	arrow.UserData.unit = unit
end

function plugin:RW_ArrowToPosition(message, id, x, y, away, extra, icon)
	if UsedArrows[id] then return end
	local arrow = self:GetArrow()
	if not arrow then return end
	
	self:InitArrow(arrow, id, away, extra, icon)
	
	arrow.UserData.x = x
	arrow.UserData.y = y
	arrow.UserData.trackType = "POSITION"
end

function plugin:RW_HideArrow(message, id)
	local arrow = UsedArrows[id]
	if arrow then
		self:HideArrow(arrow)
	end
	RW:TRemove(UsedArrowsI, arrow)
	self:ArrangeArrows()
end

function plugin:RW_HideAllArrows(message)
	for id, arrow in pairs(UsedArrowsI) do
		self:HideArrow(arrow)
	end
	wipe(UsedArrowsI)
	self:ArrangeArrows()
end

function plugin:RW_ConfigMode(message, show)
	if show then
		anchor:Show()
		self:SendMessage("RW_ArrowToPosition", "test1", 0.5, 0.5, 40, "Test1", [[Interface\PAPERDOLLINFOFRAME\UI-GearManager-Undo]])
		self:SendMessage("RW_ArrowToPosition", "test2", 0.1, 0.2, nil, "Test2", [[Interface\PAPERDOLLINFOFRAME\UI-GearManager-Undo]])
		self:SendMessage("RW_ArrowToPosition", "test3", 0.5, 0.1, nil, "Test3", [[Interface\PAPERDOLLINFOFRAME\UI-GearManager-Undo]])
	else
		anchor:Hide()
		self:SendMessage("RW_HideArrow", "test1")
		self:SendMessage("RW_HideArrow", "test2")
		self:SendMessage("RW_HideArrow", "test3")
	end
end

function plugin:SlashHandler(msg)
	self:SendMessage("RW_ArrowToNameOrUnit", "personal", nil, msg)
end

function plugin:GetOptions()
	return "Plugins", "Alert Plugins", "Direction", self.GetGUI, [[Interface\PAPERDOLLINFOFRAME\UI-GearManager-Undo]]
end

function plugin.ChangeCallback()
	plugin:Reset()
end

function plugin:GetGUI()
	local self = self or plugin
	local UIF = LibStub("AceGUI-Factory-1.0"):GetFactory("RW")
	local AceGUI = LibStub("AceGUI-3.0")
	
	local group, label, slider, check, igroup, igroup2, button, color, select
	
	group = AceGUI:Create("ScrollFrame")
	group:SetLayout("Flow")
	
	label = UIF:MainTitleLabel("Direction")
	group:AddChild(label)
	
	igroup = UIF:InlineGroup("")
	group:AddChild(igroup)	
	
	label = UIF:NormalText(
[[Arrow will guide you to or from a unit or even a set position. To open a custom arrow type |cffffff66/dir ("name")|r  Where name is the name of one player in your current party or raid]])
	label:SetImage([[Interface\PAPERDOLLINFOFRAME\UI-GearManager-Undo]])
	igroup:AddChild(label)
	
	igroup = UIF:InlineGroup("")
	group:AddChild(igroup)	
	
	check = UIF:CheckBox("Enabled", db, "PluginEnabled", function() 
		if not db.PluginEnabled then
			self:Disable()
		else
			self:Enable()
		end
	end)
	igroup:AddChild(check)
	
	local drop = UIF:Dropdown("Expand Direction")
	drop:SetList(ExpandList)
	drop:SetValue(db.ExpandDir)
	drop:SetFullWidth(true)
	drop:SetCallback("OnValueChanged", function(widget, event, value)
		db.ExpandDir = value
		plugin:Reset()
	end)
	igroup:AddChild(drop)
	
	
	color = UIF:ColorSelect("Color Near", db, "ColorClose", plugin.ChangeCallback)
	color:SetRelativeWidth(0.32)
	igroup:AddChild(color)
	
	color = UIF:ColorSelect("Color Middle", db, "ColorMiddle", plugin.ChangeCallback)
	color:SetRelativeWidth(0.32)
	igroup:AddChild(color)
	
	color = UIF:ColorSelect("Color Far", db, "ColorFar", plugin.ChangeCallback)
	color:SetRelativeWidth(0.32)
	igroup:AddChild(color)
	
	color = UIF:ColorSelect("Font Color", db, "FontColor", plugin.ChangeCallback)
	igroup:AddChild(color)
	
	slider = UIF:Slider("Font Size", db, "FontSize", 1, 40, 1, plugin.ChangeCallback)
	igroup:AddChild(slider)
	
	igroup2 = UIF:InlineGroup("Size")
	igroup:AddChild(igroup2)
	
	slider = UIF:Slider("Width", db, "Width", 10, 300, 1, plugin.ChangeCallback)
	igroup2:AddChild(slider)
	
	slider = UIF:Slider("Height", db, "Height", 10, 300, 1, plugin.ChangeCallback)
	igroup2:AddChild(slider)
	
	igroup2 = UIF:InlineGroup("Position")
	igroup:AddChild(igroup2)
		
	slider = UIF:Slider("X", db, "x",
		-floor(GetScreenWidth() * UIParent:GetEffectiveScale()), 
		floor(GetScreenWidth() * UIParent:GetEffectiveScale()),
		1, plugin.ChangeCallback)
	igroup2:AddChild(slider)
	
	slider = UIF:Slider("Y", db, "y",
		-floor(GetScreenHeight() * UIParent:GetEffectiveScale()), 
		floor(GetScreenHeight() * UIParent:GetEffectiveScale()),
		1, plugin.ChangeCallback)
	igroup2:AddChild(slider)
	
	button = UIF:Button("Reset", function() 
		plugin.db:ResetProfile() 
	end)
	button:SetUserData("confirm", true)
	button:SetUserData("TriggerGlobalUpdate", true)
	group:AddChild(button)
	
	button = UIF:Button("Hide All", function()
		plugin:SendMessage("RW_HideAllArrows")
	end)
	group:AddChild(button)
	
	return group
end